#!/usr/bin/env newlisp
;;
;; Wiki software v.4.6 for newLISP 10.0  or later
;;
;; see CHANGES file for changes  since 2.0
;; 
;;
;; This software is copyright (c) Lutz Mueller, 2008. All rights reserved.
;; It is licensed by the GNU General Public License GPL, v.3, June 2007
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; enable deletion of pages in this wiki
;; in any case pages are autmatically backed up before deletion
(set 'delete-enabled true) 
;; enable usage of the [eval: <source>] text on included pages
(set 'eval-source nil) ; v. 4.6

(set 'start-time (time-of-day))

;; following variables are global or used as globales from the dynnamic
;; scope of a function:

; body -> used in template.html for the text inside the content box
; has-comments -> used as a flag inside template.html to see if the post page as comments
; has-content-box -> used inside template.html as a flag to print content or not
; has-menu -> used as a flag inside template.html to see if the page as the index/files/etc. menu
; has-postdialog -> used as a flag inside template.html to see if the page displays a postdialog
; has-style -> used inside template.html contains the name of a page specific style sheet
; is-locked -> flag to find out if this page may be displayed
; is-protected -> flag to find out if this page is protected for editing
; page-name -> used inside template.html
; post-dialog -> contains a post dialog

(print  "Content-Type: text/html\r\n\r\n")

(load "cgi.lsp")
(load "pages/setup.lsp")

;; if no setup was done, put deafult values
(if (not SETUP:post-break) (set 'SETUP:post-break "----"))
(if (not SETUP:comment-break) (set 'SETUP:comment-break "----"))
(if (not SETUP:from-email) (set 'SETUP:from-email "wiki@site.com"))
(if (not SETUP:to-email) (set 'SETUP:to-email "name@site.com"))
(if (not SETUP:smtp) (set 'SETUP:smtp "smtp.site.com"))
(if (not SETUP:time-offset) (set 'SETUP:time-offset 0))
(if (not SETUP:security) (set 'SETUP:security "open"))
(if (not SETUP:template) (set 'SETUP:template "template.html"))
(if (not SETUP:page-title) (set 'SETUP:page-title "on"))
(if (not SETUP:toc-level) (set 'SETUP:toc-level 3))
(if (not SETUP:toc-start) (set 'SETUP:toc-start {<div class="toc"><a name="toc">Table of contents:</a><br /><ul>}))
(if (not SETUP:toc-end) (set 'SETUP:toc-end "</ul></div>"))
(if (not SETUP:toc-elm-start) (set 'SETUP:toc-elm-start "<li>"))
(if (not SETUP:toc-elm-end) (set 'SETUP:toc-elm-end "</li>"))

;; if no rss header info is supplied, make a guess
(if (not SETUP:rss-title) (set 'SETUP:rss-title (or (read-file "pages/Browser_Title") "")))
(replace {\[.*\]} SETUP:rss-title "" 512) ;; remove wiki tags
(if (not SETUP:rss-description) (set 'SETUP:rss-description SETUP:rss-title))
(set 'url (or (env "SCRIPT_URI") " "))
(if (not SETUP:rss-link) (set 'SETUP:rss-link (join (chop (parse url "/")) "/")))
(if (not SETUP:rss-copyright) 
    (set 'SETUP:rss-copyright (append "copyright (c) " (nth 2 (parse url "/")) 
            ", 2006. All rights reserved.")))

(set 'wikiword {\[\[(.*)\]\]})
(set 'valid-page {[a-zA-Z0-9 _\-#$%]+})
(set 'exclude-pattern {\!|@|\^|\&|\*|\(|\)|\{|\}|\[|\]|\||\\|\?|\"|\'|/|\+|:|;|<|>|,|\.})
(set 'is-protected nil)
(set 'is-locked nil)
(set 'has-menu true)
(set 'has-content-box true)
(set 'has-pagename true)

;; make sure page-name is syntactically correct
(define (check-page-name page-name)
;;    (if (or (not (find valid-page page-name 0)) (find " " page-name))
    (if (or (not (find valid-page page-name 0)) 
            (find " " page-name)
            (find "." page-name))
        (begin
            (set 'body (append page-name 
                  { is not a valid <a href="index.cgi?Wiki_Link">Wiki Link</a>.}))
            (CGI:put-page SETUP:template)
            (exit))))

;; make sure template-name ends in html
(define (check-template-name template)
    (if (not (ends-with template {\.html} 1))
        (begin
            (set 'body (append template 
                  { is not a valid tamplate name. Template names must end in .html .}))
            (CGI:put-page SETUP:template)
            (exit))))

;; make sure that page-name is a page for posting
(define (check-post-page page-name)
    (set 'content (get-content page-name))
    (if (not (find {\[post:(.*)\]} content 512))
        (begin
            (set 'body (append {Page: } page-name 
            { is not a valid page for posting, <a href="index.cgi">continue</a>}))
            (set 'page-name "Post")
            (CGI:put-page SETUP:template)
            (exit))
        (trim $1)))

;; extract the password from a page content
;; if [protected:] tage is present, but without
;; password then assume the master password
(define (get-password content , pass)
    (if (find {\[protected:(.*)\]} content 512) 
        (set 'pass (trim $1))
        (if (find {\[locked:(.*)\]} content 512) 
            (set 'pass (trim $1))))
    (if (= pass "") (set 'pass SETUP:master-password))
    pass)

;; check if this content has edit permission for password in key
(define (has-edit-permission content key , pass)
    (set 'pass (get-password content))
    (if
        (and (not pass) (= SETUP:security "open")) true
        (and pass (= pass key)) true
        (and key (= key SETUP:master-password)) true))

;; check if this page has view permission
;; if no password and no master-password is specified than also viewable
(define (has-view-permission content , pass)
    (if (find {\[locked:(.*)\]} content 512) 
        (set 'pass (trim $1)))
    (if (= pass "") (set 'pass SETUP:master-password))
    (not pass))

;; get the content for a page-name (file-name)
(define (get-content apage-name)
    (replace " " apage-name "_")
    (if (file? (append "pages/" apage-name))
        (read-file (append "pages/" apage-name))
            ""))

;; display a search dialog with or without pattern in search-term
(define (search-dialog search-term)
    (append
        {<form name="searchform" action="index.cgi" method="post">
    <input type="text" name="search" class="textarea" size="20" value="}
        search-term {"/>}
    {<input type="submit" name="submit" class="button" value="search"/>}
        {</form> }))

;; display edit dialog with or without content for page-name
(define (edit-dialog content page-name key)
    (append
        {<center>}
        {<form name="postform" action="index.cgi" method="post">}
        {<textarea class="textarea" rows="24" cols="100" wrap="virtual" name="content">}
        content
        {</textarea><br>}
        {<input type="hidden" name="page-name" value="} page-name {"/>}
        (if key
            (string
                {<input type="hidden" name="} "key" 
                {" value="} key {"/>}) {})
        {<input type="submit" name="postedit" class="button" value="save" />}
        {</form></center>}))

;; display a comment dialog for page-name
(define (comment-dialog page-name)
    (if (not comment-text) (set 'comment-text ""))
    (if (not comment-author) (set 'comment-author ""))
    (append {<br><br>}
        {<form name="commentform" action="index.cgi" method="post">}
        {Name: <input class="textarea" type="text" name="author" size="16" value="}
        comment-author {"/><br><br>}
        {<textarea class="textarea" rows="6" cols="72" wrap="virtual" name="comment">}
        comment-text {</textarea>}
        {<input type="hidden" name="page-name" value="} page-name {"/><br><br>}
        {<input type="submit" name="postcomment" class="button" value="post comment"/>&nbsp;&nbsp;&nbsp;}
        {&nbsp;&nbsp;<input type="submit" name="postcomment" class="button" value="preview"/>}
        {</form>}))

;; display a post dialog
(define (post-dialog)
    (if (not post-author) (set 'post-author ""))
    (if (not post-key) (set 'post-key ""))
    (if (not post-subject) (set 'post-subject ""))
    (if (not post-text) (set 'post-text ""))
    (append {<br><br>}
        {<center>}
        {<form name="postform" action="index.cgi" method="post">}
        {Name: <input class="textarea" type="text" name="author" size="16" value="}
        post-author {"/>&nbsp;&nbsp;&nbsp;}

        ;; this dialog has no password [post:], used for contact info etc.
        (if (!= post-password "")
           (append 
              {Password: <input class="textarea" type="password" name="key" size="16" value="}
              post-key {"/><br><br>})
           {<br><br>})

        {Subject: <input class="textarea" type="text" name="subject" size="40" value="}
        post-subject {"/><br><br>}
        {<textarea class="textarea" rows="6" cols="72" wrap="virtual" name="text">}
        post-text {</textarea>}
        {<input type="hidden" name="postpage" value="} post-page {"/><br><br>}
        {<input type="submit" name="postit" class="button" value="post"/>&nbsp;&nbsp;&nbsp;}
        {&nbsp;&nbsp;<input type="submit" name="postit" class="button" value="preview"/>}

    ;; comments checkbox s suppressed for post dialogs w/o password
    (if (!= post-password "")
        (append {&nbsp;&nbsp;comments: }
              (if (= commentspage "yes")
                 {<input type="checkbox" checked name="commentspage" value="yes"/>}
                 {<input type="checkbox" name="commentspage" value="yes"/>})) {})
    {</form>}
    {</center>}
))


;; display a setup dialog
(define (setup-dialog)
    (set 'templates (directory "." "\.html"))
    (if (set 'pos (find SETUP:template templates))
        (rotate templates (- 0 pos)))        
    (append
        {<HTML><BODY>}
        {<h2>Confirm or change settings</h2>}
        {<form name="setupform" action="index.cgi" method="post">}
        {Master password: <input type="text" name="newpass" class="textarea" size="20" }
        {value="} SETUP:master-password {"/><br><br>}
        {Security: <select name="security">}
            (if (= SETUP:security "closed") 
        {<option>closed<option>open}
        {<option>open<option>closed})
        {</select><br><br>}
        {Template: <select name="template">}
        (append "<option>" (join templates "<option>"))
        {</select><br><br>}
        {Page Title: <select name="pagetitle">}
            (if (= SETUP:page-title "on") 
       	        {<option>on<option>off}
                {<option>off<option>on})
        {</select><br><br>}
        {Post break: <input type="text" name="postbreak" class="textarea" size="20" }
        {value="} SETUP:post-break {"/><br><br>}
        {Comment break: <input type="text" name="commentbreak" class="textarea" size="20" }
        {value="} SETUP:comment-break {"/><br><br>}
        {Repeat full post on comments page: }
        (if (= SETUP:full-post "yes")
             {<input type="checkbox" checked name="fullpost" value="yes"/><br><br>}
             {<input type="checkbox" name="fullpost" value="yes"/><br><br>})
        {UTC offset in minutes: <input type="text" name="timeoffset" class="textarea" size="8" }
        {value="} (string SETUP:time-offset) {"/><br><br><br>}
        {Notify on post/coment: }
        (if (= SETUP:post-notify "yes")
             {<input type="checkbox" checked name="postnotify" value="yes"/><br><br>}
             {<input type="checkbox" name="postnotify" value="yes"/><br><br>})
        {from email address: <input type="text" name="fromemail" class="textarea" size="40" }
        {value="} SETUP:from-email {"/><br><br>}
        {to email address: <input type="text" name="toemail" class="textarea" size="40" }
        {value="} SETUP:to-email {"/><br><br>}
        {SMTP mail server: <input type="text" name="smtp" class="textarea" size="32" }
        {value="} SETUP:smtp {"/><br><br><br>}
        {rss title: <input type="text" name="rsstitle" class="textarea" size="50" }
        {value="} SETUP:rss-title {"/><br><br>}
        {rss description: <input type="text" name="rssdescription" class="textarea" size="50" }
        {value="} SETUP:rss-description {"/><br><br>}
        {rss and email site-link: <input type="text" name="rsslink" class="textarea" size="50" }
        {value="} SETUP:rss-link {"/><br><br>}
        {rss-copyright: <input type="text" name="rsscopyright" class="textarea" size="50" }
        {value="} SETUP:rss-copyright {"/><br><br>}
        {<input type="hidden" name="key" value="} SETUP:master-password {"/>}
        {<input type="submit" name="setupdone" class="button" value="submit" />}
        {</form></body></html>}))


;; display a password dialog
(define (password-dialog text hidden-name hidden-value)
    (if (not hidden-value) (set 'hidden-value ""))
    (append text
            {<form name="passform" action="index.cgi" method="post">}
            {<input type="password" name="key" class="textarea" size="20" />}
            {<input type="hidden" name="} hidden-name {" value="} hidden-value {"/>}
            {<input type="submit" name="submit" class="button" value="submit" />}
            {</form>}))


;; display files table
(define (files-table)
    (set 'files '({</table>}))
    (set 'cnt 0)
    (dolist (fle (sort (get-pages) '>))
        (inc cnt)
        (get-flags fle)
        (set 'dfle fle) (replace "_" dfle " ")
        (push (string {<tr>}
                  {<td>&nbsp;<a href="index.cgi?page=} 
                  fle {">} dfle {</a>&nbsp;</td>}
                  {<td>&nbsp;<a href="index.cgi?edit=} fle {">edit</a></td>}
                  {<td>&nbsp;<a href="index.cgi?rename=} dfle {">rename</a></td>}
                  {<td>&nbsp;<a href="index.cgi?backup=} fle {&continue">backup</a></td>}
                  {<td>&nbsp;<a href="index.cgi?restore=} fle {">restore</a></td>}
                  {<td>&nbsp;<a href="index.cgi?delete=} fle {&continue">delete</a></td>}
                  {<td style="color:#cc0; font-weight:bold">} L-flag {</td>}
                  {<td style="color:#c00; font-weight:bold">} P-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} M-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} T-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} S-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} O-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} C-flag {</td>}
                  {</tr>}) files))

        (push {<table style="border-width: 0px;">} files)
        (set 'body (join files "\r\n"))            
        (set 'body (append body (get-legend)
        {<br><small><font style="color:#888;">found } 
        (string cnt) { files</font></small>}))
        (set 'page-name "Manage_Pages")
        (CGI:put-page SETUP:template)
        (exit))

;; search table, make a HTML table of file/page-names with pattern
(define (search-content-table pattern)
        (set 'file-list '("</table>\r\n"))
        (dolist (fle (sort (get-pages)))
            (set 'content (get-content fle))

            ;; don't show the password option
            (replace {\[protected:.*\]} content "" 512)
            ;; don't show the post option
            (replace {\[post:.*\]} content "" 512)

            ;; don't search in locked pages
            (if (find {\[locked:.*\]} content 512) 
                (set 'pos nil)
                (set 'pos (find pattern content 0)))
            (set 'fle-name fle)
            (replace "_" fle-name " ")
            (if pos (push (append 
                {<tr><td><a href="index.cgi?page=} fle {">}
                 fle-name {</a></td></tr>} ) file-list)))
    (push {<table>} file-list)
    (join file-list "\r\n"))


(define (search-title-table pattern)
    (set 'file-list (filter (fn (fle) (find pattern (replace "_" fle " ") 0)) (get-pages)))
    (set 'files '({</table>}))
    (dolist (pname (sort file-list '>))
        (set 'fle pname)
        (push (string 
                 {<tr><td><a href="index.cgi?page=} (replace " " fle "_")
                 {">} pname {</a></td></tr>}) files))

        (push {<table style="border-width: 0px;">} files)
        (join files "\r\n"))

;; creates an empty TOC context
(define (init-toc)
  (set 'TOC:toc (list SETUP:toc-start))
  (set 'TOC:lvl 1)
  (set 'TOC:anchor 0)
  (set 'TOC:found false))


;; checks the given line and updates TOC data
(define (update-toc line)

  (define (append-toc content level)
    (unless (> level SETUP:toc-level)
      (while (> level TOC:lvl)
        (push "<ul>" TOC:toc)
        (++ TOC:lvl))
      (while (< level TOC:lvl)
        (push "</ul>" TOC:toc)
        (-- TOC:lvl))
      (push (append SETUP:toc-elm-start {<a href="#a} (string TOC:anchor) {">} content {</a>} SETUP:toc-elm-end) TOC:toc)))
    
  (cond
    ((find "======(.*)======" line 512) (append-toc $1 5))
    ((find "=====(.*)=====" line 512) (append-toc $1 4))
    ((find "====(.*)====" line 512) (append-toc $1 3))
    ((find "===(.*)===" line 512) (append-toc $1 2))
    ((find "==(.*)==" line 512) (append-toc $1 1))
    ((find {\[toc\]} line 512) (set 'TOC:found true)) ))

;; returns an anchored content for TOC, if needed
;; this function also updates the anchor number
(define (get-anchor content level)
  (if (> level SETUP:toc-level)
    content
    (let ( str  (append {[TOC]<a name="a} (string TOC:anchor) {" href="#toc">[TOC]} content {[TOC]</a>[TOC]}) )
      (++ TOC:anchor)
      str)))

;; applies all the changes needed to properly display (or not) TOC
(define (finalize-toc)
  (if TOC:found
    (begin
      (push SETUP:toc-end TOC:toc)
      (reverse TOC:toc)
      (let ( str-toc  (join TOC:toc "") )
        (replace {\[toc\]} page str-toc 512)
        (replace {\[TOC\]</a>\[TOC\]} page "</a>" 512)
        (replace {\[TOC\]<a name(.*)\[TOC\]} page (append "<a name" $1) 512)))
    (begin
      (replace {\[TOC\]</a>\[TOC\]} page "" 512)
      (replace {\[TOC\]<a name.*\[TOC\]} page "" 512))))

;; format a page line by line, skip comments and monospaced lines
(define (format-page pge , page)
  (set 'page '())
  (init-toc)
  (dolist (str (parse pge "\r\n"))
    (if (starts-with str " ")
      (begin
        (replace "&" str "&amp;")
        (replace "<" str "&lt;")
        (replace ">" str "&gt;")
        (replace " " str "&nbsp;")
        (push (append "<tt>" (slice str 6) "</tt>") page))
        ;; supress lines, which are only comments
        (if (not (find "<!--.*-->" str 512))
          (begin
            (update-toc str)
            (push (format-line str) page)))))
  (set 'page (join (reverse page) "<br>\r\n"))
  (replace "> <br>\r\n" page ">\r\n") 
  (replace "a> ." page "a>.") 
  (replace "a> ," page "a>,") 
  (replace "a> ;" page "a>;") 
  (replace "a> :" page "a>:") 
  (replace "a> !" page "a>!") 
  (replace "a> ?" page "a>?")
  (finalize-toc)
)

;; hide second level includes and locked pages
(define (format-include page , content params)
	(if (find "?" page) ; does pagename have a URL query string
		(begin
			(set 'content (get-content  (first (parse page "?"))))
			(set 'params (parse (last (parse page "?")) "&"))
			(dolist (item params)
				(set 'var (first (parse item "=")))
				(set 'value (CGI:url-translate (last (parse item "=")))	)
				(replace (append "%" var "%") content value)
				(replace (append "%" var "=(.*)%") content value 512)
				))
    		(set 'content (get-content page)))
	; replace un-called templates with defaults
    (replace "%(.*)=(.*)%" content $2 512)
    (if eval-source
    	(replace {\[eval:(.*)\]} content (string (eval-string $1 'User (error-text))) 516))
    (if (find {\[locked:.*\]} content 512) ""
        (replace {\[include:.*\]} (format-page content) " " 512))
)

;; create tagsearch links
(define (make-tagsearch tagstr, taglst)
  (when (setq taglst (map trim (parse tagstr ",")))
        (join (map (fn (s) (string {<a href="index.cgi?tagsearch=} s {">} s {</a>})) taglst) ", ")))

;; format a line
(define (format-line str)

    ;; protect legitimate HTML tags
    (replace "<(br|p|ul|ol|li|table|th|tr|td|center|blockquote|i|em|b|u|tt|sup|sub|small|big)>" 
              str (append "[" $1 "]") 0)
    (replace "</(br|p|ul|ol|li|table|th|tr|td|center|blockquote|i|em|b|u|tt|sup|sub|small|big)>" 
              str (append "[/" $1 "]") 0)

    ;; hide comments
    (replace {<\!--.*-->} str "" 512)

    ;; replace forbidden characters
    (replace "<" str "&lt;")
    (replace ">" str "&gt;")
    ;; for handling line breaks
    (replace "\r\n" str " \r\n") 

    ;; put back legitimate HTML tags
    (replace {\[(i|em|b|u|tt|sup|sub|small|big)\]} str (append "<" $1 ">") 0)
    (replace {\[(br|p|ul|ol|li|table|th|tr|td|center|blockquote)\]} str (append "<" $1 ">") 0)

    (replace {\[/(i|em|b|u|tt|sup|sub|small|big)\]} str (append "</" $1 ">") 0)
    (replace {\[/(br|p|ul|ol|li|table|th|tr|td|center|blockquote)\]} str (append "</" $1 "> ") 0)

    ;; insert search dialog
    (replace "[search:]" str (search-dialog ""))

    ;; don't show the password option
    (replace {\[protected:.*\]} str "" 512)

    ;; don;t show the menu option
    (replace {\[nomenu:\]} str "" 512)

    ;; don;t show the locked option
    (replace {\[locked:.*\]} str "" 512)

    ;; don't show style option
    (replace {\[style:.*\]} str "" 512)

    ;; don't show noname / notitle option
    ;; noname is deprecated
    (replace {\[(noname|notitle):\]} str "" 512)

    ;; don't show comments option
    (replace {\[comments:\]} str "" 512)

    ;; get comments count from comments page
    ;; if comment page does not exist, create it
    (replace {\[comments-count:(.*)\]}  str (comment-count (trim $1)) 512)

    ;; don't show post option
    (replace {\[post:.*\]} str "" 512)

    ;;; create tag links
    (replace {\[tags:(.*)\]} str (string "<div class=\"toc\">Tags: " (make-tagsearch $1) "</div>") 512)

    ;; system variable with page title
    (replace "%this%" str page-name)

    ;; generate HTML for easy titles
    (replace "======(.*)======" str (append "<h5>" (get-anchor $1 5) "</a></h5> ") 512)
    (replace "=====(.*)=====" str (append "<h4>" (get-anchor $1 4) "</h4> ") 512)
    (replace "====(.*)====" str (append "<h3>" (get-anchor $1 3) "</h3> ") 512)
    (replace "===(.*)===" str (append "<h2>" (get-anchor $1 2) "</h2> ") 512)
    (replace "==(.*)==" str (append "<h1>" (get-anchor $1 1) "</h1> ") 512)

    ;; generate HTML for indented blocks
    (if (starts-with str ": ")
        (set 'str (append "<blockquote>" (slice str 2) "</blockquote>")))

    ;; generate HTML for bullets
    (set 'spaces "&nbsp;&nbsp;&nbsp;&nbsp;")
    (replace {\*\*\*\*(.*)} str (append  spaces spaces spaces "*" $1 " ") 0)
    (replace {\*\*\*(.*)} str (append  spaces spaces "*" $1 " ") 0)
    (replace {\*\*(.*)} str (append  spaces "*" $1 " ") 0)
    (replace {\*(.*)} str (append "&#149;" $1 " ") 0)

    ;; generate HTML for bold italics
    (replace {\'\'\'\'\'(.*)\'\'\'\'\'} str (append "<b><i>" $1 "</i></b>") 512)

    ;; generate HTML for bold
    (replace {\'\'\'(.*)\'\'\'} str (append "<b>" $1 "</b>") 512)

    ;; generate HTML for italics
    (replace {\'\'(.*)\'\'} str (append "<i>" $1 "</i>") 512)

    ;; generate a horizontal ruler
    (replace "----" str "<hr> ")

    ;; find wiki links and replace with HTML links
    (replace wikiword str
    (if (file? (append "pages/" (replace " " (set 'fle $1) "_")))
        (append  {<a href="index.cgi?page=} fle {">} $1 {</a> })
        (append  {<a href="index.cgi?edit=} fle {">??? } $1 {</a> }) ) 512)

    ;; find image tags
    (replace {\[image:(.*) (left|right|top|middle|bottom) (.*)\]} str
        (append {<img src="} $1 {" alt="} $3 {" align="} $2 {" order="0" border="0"/>}) 512)
    (replace {\[image:(.*) (.*)\]} str
        (append {<img src="} $1 {" alt="} $2 {" order="0" border="0"/>}) 512)

    ;; find link reference with extra text and translate to HTML
    (replace {\[http:(.*) (.*)\]} str 
      (if (starts-with $1 "//")
    (append {<a href="http:} $1 {">} $2 {</a> })
    (append {<a href="} $1 {">} $2 {</a> }))
      512)



    ;; generate custom css span style, may not contain other [,] tags
    (replace {\[span:(.*) (.*)\]} str (append {<span class="} $1 {">} $2 {</span>}) 512)

    ;; quick tables
    (if (= str "[") 
        (begin
            (set 'str {<table class="quick"> })
            (set 'is-in-table true)))
    (if (= str "]") 
        (begin
            (set 'str "</table> ")
            (set 'is-in-table nil)))
    (if is-in-table 
      (begin
    (if (starts-with str "|||||") 
      (set 'str (append {<tr class="quick"><td class="quick" colspan="4" valign="top">} (slice str 5))))
    (if (starts-with str "||||") 
      (set 'str (append {<tr class="quick"><td class="quick" colspan="3" valign="top">} (slice str 4))))
    (if (starts-with str "|||") 
      (set 'str (append {<tr class="quick"><td class="quick" colspan="2" valign="top">} (slice str 3))))
    (if (starts-with str "||") 
      (set 'str (append {<tr class="quick"><td class="quick" valign="top">} (slice str 2))))
    (if (ends-with str "||") 
      (set 'str (append (slice str 0 (- (length str) 2)) "</td></tr> ")))
    (replace {|||||} str {</td><td class="quick"  colspan="4" valign="top">})
    (replace {||||} str {</td><td class="quick"  colspan="3" valign="top">})
    (replace {|||} str {</td><td class="quick"  colspan="2" valign="top">})
    (replace {||} str {</td><td class="quick" valign="top">})))


    ;; generate custom css div style, may not contain other [,] tags
    (replace {\[div:(.*) (.*)\]} str (append {<div class="} $1 {">} $2 {</div>}) 512)

    str)


;; except for [http:...] and [image:...] tags only
;; character formatting without special
;; strings is allowed
(define (sanitize-post post image-flag)
    (replace "&" post "&amp;")
        (replace "<" post "&lt;")
        (replace ">" post "&gt;")
    ;; protect links and images
    (replace {\[http:(.*) (.*)\]} post (append {%http:} $1 { } $2 {%}) 512)
    (if image-flag (begin
        (replace {\[image:(.*) (left|right|top|middle|bottom) (.*)\]} post
        (append {%image:} $1 { } $2 { } $3 {%}) 512)
        (replace {\[image:(.*) (.*)\]} post (append {%image:} $1 { } $2 {%}) 512)))
        (replace "[" post "&#091")
    (replace "]" post "&#093")
    ;; put link back and images back
    (replace {%http:(.*) (.*)%} post (append {[http:} $1 { } $2 {]}) 512)
    (if image-flag (begin
        (replace {%image:(.*) (left|right|top|middle|bottom) (.*)%} post 
        (append {[image:} $1 { } $2 { } $3 {]}) 512)
        (replace {%image:(.*) (.*)%} post (append {[image:} $1 { } $2 {]}) 512)))
    post )


;; compose a post/comment header from auhtor and time of day
(define (get-header author post-time , header link)
    (set 'header (append author " on " 
             (date (apply date-value post-time) (int SETUP:time-offset 0)) ))
    (set 'link header)
    (replace " " link "%20")
    (set 'header (append "\r\n====== " header "[http:index.cgi?find=" link "  #]"
            " ======\r\n" )))

;; compose post/comment signature from the remote IP address
(define (get-signature)
    (set 'ip (or (env "REMOTE_ADDR") (env "REMOTE_HOST") ""))
    (if (= ip "") (set 'ip "0.0.0.0"))
    (append "<!--" ip "-->"))

;; compose a comment link/counter from time of day
;; if a comments page does not exist yet create it using the postime
;; and post subject or body as title/top of the page
(define (comment-link post-time subject)
    (if (= SETUP:full-post "yes")
    (set 'fmt "P-%4d-%02d-%02d-%02d-%02d-%02d-%06d") 
    (set 'fmt "C-%4d-%02d-%02d-%02d-%02d-%02d-%06d"))
    (set 'page (eval (append '(format fmt) (slice post-time 0 7))))
    (create-comments-page page subject)
    (append "<small>[http:index.cgi?" page " comments ([comments-count: " page "])]</small>"))

;; create a comments page with title from subject
(define (create-comments-page comment-page subject)
    (if (!= SETUP:full-post "yes") ;; display subject line only
    (set 'subject (append "===== " subject " =====\r\n")))
    (replace exclude-pattern comment-page "" 0)
    (replace " " comment-page "_")
    (if (not (file? (append "pages/" comment-page)))
        (write-file (append "pages/" comment-page) 
                    (append "[notitle:]\r\n" subject "[comments:]\r\n"))))

;; count comments on a comments page
(define (comment-count comment-page , fle cnt)
    (replace exclude-pattern comment-page "" 0)
    (replace " " comment-page "_")
    (if (not (file? (append "pages/" comment-page)))
        (write-file (append "pages/" comment-page) 
                            "[notitle:][comments:]\r\n"))
    (set 'fle (open (append "pages/" comment-page) "read"))
    (set 'cnt 0)
    (while (set 'line (read-line fle))
        (if (find {<\!--.*\..*\..*\..*-->} line 0) (inc cnt)))
    (close fle)
    (string cnt))
           

;; display a page from page-name/file-name
;; resolve includes for one level
(define (display-page page-name)
    (check-page-name page-name)
    (if (file? (append "pages/" page-name))
        (begin
            (set 'body (get-content page-name))
            (get-display-flags body)
            (if (not is-locked) 
                (begin
                   (set 'body (format-page body))
                   (replace {\[include:(.*)\]} body (format-include (trim $1)) 512)
                   (replace {\[search:.(.*)\]} body (search-content-table $1) 512)
                   (replace {\[title:.(.*)\]} body (search-title-table $1) 512))
                (set 'body (password-dialog 
                             {Enter password to unlock page for viewing:<br><br>}
                             "unlock" page-name))))

        (set 'body (append {This page does not exist, click <a href="index.cgi?edit=} 
                           page-name {">here</a>} { to create it.})))

    (CGI:put-page SETUP:template))


;; count the pages which refer to a page
(define (refcount page , files cnt content)
    (set 'page (replace "_" page "."))
    (set 'files (slice (directory "pages/") 2))
    (set 'cnt 0)
    (dolist (fle files)
        (set 'content (get-content fle))
        (if (find page content 0) (inc cnt)))
    cnt)

;; changes all references to the given page-old
;; to references to the given page-new
(define (change-refs page-old page-new)
  (let ( page-old-filename  (replace "_" (dup page-old 1) ".")
         rexp               (append {\[\[} page-old {\]\]})
         subst              (append {[[} page-new {]]})
         files              (slice (directory "pages/") 2) )
    (dolist (fle files)
      (let ( content  (get-content fle) )
        (if (find rexp content 0)
          (begin
            (replace rexp content subst 512)
            (write-file (append "pages/" fle) content)))))))

;; backup a spefic page, postfix the name with current time
(define (backup fname , date-str)
    (set 'date-str (eval 
        (append '(format "%4d%02d%02d-%02d-%02d-%02d-%06d") (slice (now) 0 7))))
    (copy-file (append "pages/" fname) (append "backup/" fname "." date-str)))

;; restore a specific page, by displaying a list of backups 
(define (restore fname , files date-str)
    (set 'files (sort 
    (filter (fn (n) (starts-with n (append fname "."))) (directory "backup/"))))
    (set 'table  '("</table>"))
    (dolist (fle files)
       (set 'date-str (last (parse fle ".")))
       (set 'date-str (append (slice date-str 0 4) " "
                              (slice date-str 4 2) " "
                              (slice date-str 6 2) " "
                              (slice date-str 9 2) ":"
                              (slice date-str 12 2) ":"
                              (slice date-str 15 2) "."
                  (slice date-str 18)))
       (push (string
                {<tr><td><a href="index.cgi?recover=} 
                fle {">} date-str {</a> </td></tr>})
             table) )
    (push {<table style="border-width: 0px;">} table)
    (if files
        (set 'body (append "Click date to restore.<br><br>\r\n" (join table "\r\n")) )
        (set 'body "Nothing to recover, no backups for this page."))
    (set 'page-name (string "Restore page: " fname))
    (CGI:put-page SETUP:template)
    (exit))
    
    
;; return a list of all pages
(define (get-pages , files)
    (difference (directory "pages/") '("." ".." ".htaccess" "setup.lsp")))

;; resturn a list of all backup files
(define (get-backup , files)
    (set 'files (slice (directory "backup/") 2))
    (replace ".htaccess" files))

;; compose a legend for the status flags
(define (get-legend)
    (append {<br>}
        {<table><tr>}
        {<td><font style="color:#cc0; font-weight:bold">&nbsp;Locked&nbsp;</font></td>}
        {<td><font style="color:#c00; font-weight:bold">&nbsp;Protected&nbsp;</font></td>}
        {<td><font style="color:#080; font-weight:bold">&nbsp;no Menu&nbsp;</font></td>}
        {<td><font style="color:#080; font-weight:bold">&nbsp;no Title&nbsp;</font></td>}
        {<td><font style="color:#080; font-weight:bold">&nbsp;Style&nbsp;</font></td>}
        {<td><font style="color:#080; font-weight:bold">&nbsp;pOsts&nbsp;</font></td>}
        {<td><font style="color:#080; font-weight:bold">&nbsp;Comments&nbsp;</font></td>}
        {</tr></table>}))

;; get all the flags for a page/file
(define (get-flags fle , content)
    (set 'content (get-content fle))
    (if (find {\[locked:.*\]} content 512) 
        (set 'L-flag "L") (set 'L-flag "&nbsp;"))
    (if (or (find {\[protected:.*\]} content 512) (= SETUP:security "closed"))
        (set 'P-flag "P") (set 'P-flag "&nbsp;"))
    (if (or (find {\[nomenu:\]} content 512) (= SETUP:security "closed"))
        (set 'M-flag "M") (set 'M-flag "&nbsp;"))
    (if (or (find {\[(notitle|noname):\]} content 512) (= SETUP:security "closed"))
        (set 'T-flag "T") (set 'T-flag "&nbsp;"))
    (if (find {\[style:.*\]} content 512) 
        (set 'S-flag "S") (set 'S-flag "&nbsp;"))
    (if (find {\[post:.*\]} content 512) 
        (set 'O-flag "O") (set 'O-flag "&nbsp;"))
    (if (find {\[comments:\]} content 512) 
        (set 'C-flag "C") (set 'C-flag "&nbsp;"))
)

;; get the display flags for a content
(define (get-display-flags abody)
    (set 'is-protected (not (has-edit-permission abody)))
    (set 'is-locked (not (has-view-permission abody)))
    (set 'has-menu (and (not (find {\[nomenu:} abody 512)) (= SETUP:security "open")))
    (set 'has-pagename (and (not (find {\[(notitle|noname):\]} abody 512)) (not (= SETUP:page-title "off"))))
    (set 'has-comments (find {\[comments:\]} abody 512))
    (set 'has-style (find {\[style:(.*)\]} abody 512))
    (if has-style 
        (begin
            (set 'has-style  (trim $1) ) 
            (replace " " has-style "_")))
)

;; strip all flags from content
(define (strip-flags content)
        (replace {\[.*:.*\]} content "" 512))

;; show a list of all files for function 
(define (show-file-list function , file-list)
    (if (= function "recover")
        (set 'file-list (get-backup))
        (set 'file-list (get-pages)))

    (set 'files '({</table>}))
    (dolist (fle (sort file-list '>))
      (if (= function "recover")
        (push (string {<tr><td>}
                  {&nbsp;<a href="index.cgi?} function {=} fle {">} 
                  (last (parse fle ".")) {</a>}
                  {&nbsp;</td><td>}
                  {&nbsp;<a href="index.cgi?} function {=} fle {">} 
                  (replace "_" (first (parse fle ".")) " ") {</a>}
                  {&nbsp;</td></tr>}) files)
                  
        (push (string {<tr><td>}
                  {&nbsp;<a href="index.cgi?} function {=} fle {">} (replace "_" fle " ") {</a>}
                  {&nbsp;</td></tr>}) files)))

    (push {<table style="border-width: 0px;">} files)
    (set 'body (join files "\r\n"))            
    (if (not page-name) 
        (set 'page-name (append "Select a file for function " function)))
        (CGI:put-page SETUP:template)
        (exit))
    

(define (notify-mail post-time post-author post-subject , message)
    (set 'link (append post-author " on " 
             (date (apply date-value post-time) SETUP:time-offset) ))
    (replace " " link "%20")
    (if (ends-with SETUP:rss-link "/") (set 'SETUP:rss-link (chop SETUP:rss-link)))

    (if post-subject
        (set 'message (append 
            {post by } post-author { about } post-subject ":\r\n\r\n"
            SETUP:rss-link {/index.cgi?find=} link "\r\n" ))
        (set 'message (append
            {comment by } post-author ":\r\n\r\n"
            SETUP:rss-link {/index.cgi?find=} link "\r\n" )))

    (if (load "smtp.lsp")
        (SMTP:send-mail SETUP:from-email SETUP:to-email 
            {post/comment on wiki} message SETUP:smtp)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; main entry point
;;

(set 'title (format-page (get-content "Title_Text")))
(set 'copyright (format-page (get-content "Copyright_Text")))
(set 'browser-title (format-page (get-content "Browser_Title")))

;; request to edit a page, if no page name given display list of pages
(if (CGI:get "edit")
    (begin
        (set 'file-name (CGI:get "edit"))
        (if (= file-name "") (files-table))
        (check-page-name file-name)
        (set 'content (get-content file-name))
        (replace "&" content "&amp;")
        (get-display-flags content)
        (set 'has-comments nil) ;; supress comments dialog
        (if (has-edit-permission content (CGI:get "key"))
            (begin
              (set 'has-content-box nil)
              (set 'has-pagename true)
              (set 'body (edit-dialog content file-name (CGI:get "key")))
              (set 'page-name file-name)
              (CGI:put-page SETUP:template)
              (exit)))

    (set 'body (password-dialog {Page is protected or locked, enter password:<br><br>}
        "edit" file-name))

    (set 'page-name "Edit page")
    (CGI:put-page SETUP:template)
    (exit)))


;; request to post the edit
(if (CGI:get "postedit")
    (begin
        (set 'page-name (CGI:get "page-name"))
        (check-page-name page-name)
        (set 'content (CGI:get "content"))
        (set 'old-content (get-content page-name))
        (if (has-edit-permission old-content (CGI:get "key"))
            (write-file (append "pages/" page-name) content))
        (display-page page-name)
        (exit)))


;; show template names to edit
(if (CGI:get "edittemplate")
    (begin
        (set 'file-name (CGI:get "edittemplate"))
        (if (empty? file-name) (set 'file-name SETUP:template))
        (set 'templates (directory "." "\.html"))
        (if (set 'pos (find file-name templates))
            (rotate templates (- pos))        
            (if (!= "" file-name) (push file-name templates)))
        (println {<form action="index.cgi" method="POST">})
        (println {<h2>Select template to edit</h2>})
        (println {<select name="templatename">})
        (dolist (fle templates)
            (println {<option>} fle))
        (println {</select><br /><br />})
        (if (!= SETUP:security "open") 
            (println {Password:<br /><input class="textarea" type="password" name="key" size="16"}))
        (println {<p><input type="submit" name="edittemplaterequest" class="button" value="Submit" /></p>})
        (println {<p><a href="http:index.cgi">continue</a></p>})
        (println {</form>})
        (exit))
    )


;; request for a template to edit, show the editor with template
(if (CGI:get "edittemplaterequest")
    (begin
	(if (and (!= SETUP:security "open") (!= SETUP:master-password (CGI:get "key")))
            (begin
                (println "<h2>Wrong password!</h2>"
                 {<a href="index.cgi?edittemplate">continue</a>})
                (exit)))

        (set 'template-name (CGI:get "templatename") )
        (if (file? template-name) (set 'content (read-file template-name)))
        (if content
            (begin
               (replace "&" content "&amp;")
               (replace "<" content "&lt;")
               (replace ">" content "&gt;")
               (replace "\r\n" content "\n"))
            (set 'content ""))

        (println {<head><title>Edit Template</title>}
                 {<style>}
                 ".editbox { font-family: Monaco, Courier, sans-serif;"
                 "    font-size: 0.8em; width: 100%; height: 40em;}"
                 {</style></head>}
                 {<body bgcolor="#FFFFFF">}
                 {<h2>Edit Template</h2>}
                 {<form action="index.cgi" method="post">}
                 {<textarea name="source" class="editbox">} content {</textarea>}
                 {<input type="submit" name="templatecancel" value="Cancel">}
                 {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}
                 {<input type="submit" name="templatesubmit" value="Save As">}
                 {&nbsp;&nbsp;&nbsp;&nbsp;file:&nbsp;}
                 {<input type="text" name="templatename" size="24" value="}
                 (CGI:get "templatename") {">}
                 {<input type="hidden" name="key" value="} SETUP:master-password {">}
                 {</form></body>})
        (exit)
))


;; receive edited template for saving
(if (CGI:get "templatesubmit")
    (begin
        (if (!= SETUP:master-password (CGI:get "key"))
            (begin
                (println "Wrong password!")
            (exit)))
        (set 'file-name (CGI:get "templatename") )
        (if (= (trim file-name) "") (set 'file-name "noname.txt"))
        (set 'source-txt (CGI:get "source"))
        (if (< (& 0xf (sys-info -1)) 6)
                (replace "\r\n" source-txt "\n"))
        (write-file file-name source-txt)
        (println "<h2>File " file-name " saved</h2>"
                 {<a href="index.cgi?edittemplate=} file-name 
                 {">continue</a>})
        (exit)))


;; cancel template editing
(if (CGI:get "templatecancel")
    (begin
        (println "<h2>Template editing cancelled</h2>"
                 {<a href="index.cgi">continue</a>}) 
	(exit)))

    
;; display a post dialog for a specific post-page
(if (CGI:get "post")
    (begin
        (set 'post-page (string (CGI:get "post")))
        (set 'post-subject (CGI:get "subject"))
        (if (not post-subject) (set 'post-subject ""))
        (set 'post-password (check-post-page post-page))
        (set 'page-name (append "Post on: " post-page))
        (set 'body "")
        (set 'commentspage "yes")
        (set 'has-postdialog true)
        (CGI:put-page SETUP:template)
        (exit)))

;; post the post create commentspage etc.
(if (CGI:get "postit")
    (begin
        (set 'post-author (trim (CGI:get "author")))
        (if (= post-author "") (set 'post-author "anonymous"))
        (set 'post-key (CGI:get "key"))
        (if post-key 
            (set 'post-key (trim post-key))
            (set 'post-key ""))
        (set 'post-subject (CGI:get "subject"))
        (set 'post-text (sanitize-post (CGI:get "text") true))
        (set 'post-page (CGI:get "postpage"))
        (set 'post-password (check-post-page post-page))
        (set 'commentspage (CGI:get "commentspage"))
        (if (= post-subject "")
            (begin
                (set 'page-name (append "Post on: " post-page))
                (set 'body {Please enter a subject})
                (set 'post-key "")
                (set 'has-postdialog true)
                (CGI:put-page SETUP:template)
                (exit)))
        (if (not (or (and 
            (find {\[post:(.*)\]} (get-content post-page) 512)
            (or (= (trim $1) post-key) (= post-key SETUP:master-password)))
                     (= (CGI:get "postit") "preview")))
          ;; wrong password
          (begin
            (set 'page-name (append "Post on: " post-page))
            (set 'body {Enter correct password})
            (set 'post-key "")
            (set 'has-postdialog true)
            (CGI:put-page SETUP:template)
            (exit))
          ;; password is fine
          (begin
           (set 'post-time (now))
           (set 'header (get-header post-author post-time))
           (set 'body (append "===== " post-subject " =====" header  post-text "\r\n"))
           (if (= (CGI:get "postit") "preview")
             (begin
                (set 'page-name (append "Preview post on: " post-page))
                (if (= commentspage "yes")
                    (set 'body (format-page (append body "[http:  comments(0)]\r\n" SETUP:post-break)))
                    (set 'body (format-page (append body SETUP:post-break))))
                (set 'post-key "")
                (set 'has-postdialog true)
                (CGI:put-page SETUP:template)
                (exit))
             (begin
                (if (= SETUP:full-post "yes")
                   (set 'comments-top body)
                   (set 'comments-top post-subject))
                (if (= commentspage "yes")
                   (set 'link (append (comment-link post-time comments-top)"\r\n"))
                   (set 'link "\r\n"))
                (set 'content (get-content post-page))
                (replace {\[post:.*\]} content 
                    (append $0 "\r\n" body link (get-signature) "\r\n" SETUP:post-break) 512)
                (write-file (append "pages/" post-page) content)
                (if SETUP:post-notify (notify-mail post-time post-author post-subject ))
                (set 'post-author "")
                (set 'post-subject "")
                (set 'post-text "")
                (display-page post-page)
                (exit)))
           ))))


;; request to post a comment
(if (CGI:get "postcomment")
    (begin
        (set 'page-name (CGI:get "page-name"))
        (set 'content (get-content page-name))
        (if (not (find {\[comments:\]} content 512))
            (begin
                (println "Not a comment page!")
                (exit)))
        (check-page-name page-name)
        (set 'comment-text (sanitize-post (CGI:get "comment")))
        (set 'comment-author (trim (CGI:get "author")))
        (if (= comment-author "") (set 'comment-author "anonymous"))
        (set 'comment-time (now))
        (set 'header (get-header comment-author comment-time))

        (if (= (CGI:get "postcomment") "preview")
            (begin
              (set 'body (append "===== Preview comment =====\r\n\r\n----" header comment-text "\r\n----"))
              (set 'body (format-page body))
              (set 'has-comments true)
              (set 'has-menu nil)
              (set 'has-pagename nil)
              (CGI:put-page SETUP:template)
              (exit)))

        (replace {\[comments:\]} content 
        (append  SETUP:comment-break
            header comment-text "\r\n" (get-signature) "\r\n" $0) 512)

        (write-file (append "pages/" page-name) content)
        (if SETUP:post-notify (notify-mail comment-time comment-author))
        (set 'comment-author "")
        (set 'comment-text "")
        (display-page page-name)
        (exit)))


;; request for backup, if no page is given display a list
(if (CGI:get "backup")
    (begin
        (set 'file-name (CGI:get "backup"))
        (if (= file-name "") (files-table))
        (check-page-name file-name)
        (if (has-edit-permission (get-content file-name) (CGI:get "key"))
            (begin
                (backup file-name)
                (if (CGI:get "continue") 
                    (files-table)
                    (display-page file-name))
            (exit)))
        (set 'body (password-dialog {Page is protected or locked, enter password:<br><br>}
            "backup" file-name))
        (set 'page-name "Backup page")
        (CGI:put-page SETUP:template)
        (exit)))


;; request for restore if no page is gicen display list of
;; all backup files which could be recovered
(if (CGI:get "restore")
    (begin
        (set 'file-name (CGI:get "restore"))
        (if (= file-name "") (files-table))
        (check-page-name file-name)
        (restore file-name)
        (display-page file-name)
        (exit)))


;; recover a backup file, if no filename given dislplay a list
(if (CGI:get "recover")
    (begin
        (set 'file-name (CGI:get "recover"))
        (if (= file-name "")
            (show-file-list "recover"))
        (set 'target (first (parse file-name ".")))
        (check-page-name target)
        (if (has-edit-permission (get-content target) (CGI:get "key"))
            (begin
                (copy-file (append "backup/" file-name) (append "pages/" target))
                (display-page target)
                (exit)))
        (set 'body (password-dialog {Page is protected or locked, enter password:<br><br>}
            "recover" file-name))
        (set 'page-name "Recover page")
        (CGI:put-page SETUP:template)
        (exit)))


;; request to delete (first backup), if no page is given show a list of pages
(if (CGI:get "delete")
    (begin
        (if (not delete-enabled) 
            (begin
                (display-page "Home")
                (exit)))
        (set 'file-name (CGI:get "delete"))
        (if (= file-name "") (files-table))
        (check-page-name file-name)
        (if (has-edit-permission (get-content file-name) (CGI:get "key"))
          (begin
            (backup file-name)    
            (delete-file (append "pages/" file-name))
            (if (CGI:get "continue") 
                (files-table)
                (display-page "Home"))
            (exit)))
        (set 'body (password-dialog {Page is protected or locked, enter password:<br><br>}
          "delete" file-name))
        (set 'page-name "Delete page")
        (CGI:put-page SETUP:template)
        (exit)))


;; request to backup all pages, requires master-bassword
;; if no master-password given will backup anyway
(if (CGI:get "backupall")
  (begin 
    (if (= (CGI:get "key") SETUP:master-password)
        (begin
            (set 'cnt 0)
            (dolist (fle (get-pages))
                (backup fle)
                (inc cnt))
            (set 'body (string  {<br>backed up } cnt 
                { files: <a href="index.cgi">continue</a>})))
        (set 'body (password-dialog {Enter the master password:<br><br>} "backupall")))
    (set 'page-name "Backup all pages")
    (CGI:put-page SETUP:template)
    (exit)))


;; request to delete all backup files and make fresh backup
;; of all files
(if (CGI:get "refreshall")
  (begin 
    (if (= (CGI:get "key") SETUP:master-password)
        (begin
            ;; delete entire backup
            (dolist (fle (get-backup))
                (delete-file (append "backup/" fle)))
            ;; backup all files
            (set 'cnt 0)
            (dolist (fle (get-pages))
                (backup fle)
                (inc cnt))

            (set 'body (string  {<br>refreshed backup for } cnt 
                { files: <a href="index.cgi">continue</a>})))
         (set 'body (password-dialog {Enter the master password:<br><br>} "refreshall")))
    (set 'page-name "Refresh backup")
    (CGI:put-page SETUP:template)
    (exit)))


;; request to unlock/display a page
;; checks for password then displays page
(if (CGI:get "unlock")
    (begin
        (set 'page-name (CGI:get "unlock"))
        (set 'content (get-content page-name))
        (if (find {\[locked:(.*)\]} content 512) (set 'locked-pass (trim $1)))
        (if (= locked-pass "") (set 'locked-pass SETUP:master-password))
        (get-display-flags content)
        (replace "&" content "&amp;")
        (if (or 
              (= locked-pass (trim (CGI:get "key")))
              (= (trim (CGI:get "key")) SETUP:master-password))
        (begin
            (set 'body (format-page content))
            (CGI:put-page SETUP:template)
            (exit))
        (begin
            (display-page page-name)
            (exit)))))


;; request for setup,  displays password dialog
(if (CGI:get "setup")
    (begin
    (if SETUP:master-password
        (set 'text "Enter current master password for setup")
        (set 'text "Enter a new master password for setup"))

    (set 'body (append
        {<html></body>}
        {<H2>} text {</h2>}
        {<form name="passform" action="index.cgi" method="post">}
        {<input type="password" name="master" class="textarea" size="20" />}
        {<input type="submit" name="submit" class="button" value="submit" />}
        {</form></body></html>}))
    (println body) 
    (exit)))


;; accepts/confirms a new password
;; if password is accepted display setup dialog
;; if password is new display password confirmation dialog
(if (CGI:get "master")
    (begin
        (if SETUP:master-password
        (if (= SETUP:master-password (CGI:get "master"))
        (set 'body (setup-dialog))
        (set 'body (append
            {The password entered is not correct, setup cannot be done.<br><br>}
            {<a href="index.cgi">continue</a>})))
            ;; no password has been saved yet
            (begin
              (set 'body (append
                {Confirm or change new password<br><br>}
                {<form name="passform" action="index.cgi" method="post">}
                {<input type="text" name="newpass" class="textarea" size="20" }
                {value="}  (CGI:get "master") {"</>}
                {<input type="submit" name="setupdone" class="button" value="submit" />}
                {</form>}))))

    (set 'page-name "Setup")
    (println body)
    (exit)))

;; accept setup form data
(if (and (CGI:get "setupdone") (= (CGI:get "key") SETUP:master-password))
    (begin
        (set 'SETUP:master-password (trim (CGI:get "newpass")))
        (set 'SETUP:security (CGI:get "security"))
        (set 'SETUP:template (CGI:get "template"))
        (set 'SETUP:page-title (CGI:get "pagetitle"))
        (set 'SETUP:post-break (CGI:get "postbreak"))
        (set 'SETUP:comment-break (CGI:get "commentbreak"))
        (set 'SETUP:full-post (CGI:get "fullpost"))
        (set 'SETUP:time-offset (int (CGI:get "timeoffset") 0))
        (set 'SETUP:post-notify (CGI:get "postnotify"))
        (set 'SETUP:from-email (CGI:get "fromemail"))
        (set 'SETUP:to-email (CGI:get "toemail"))
        (set 'SETUP:smtp  (CGI:get "smtp"))
        (set 'SETUP:rss-title (CGI:get "rsstitle"))
        (set 'SETUP:rss-description (CGI:get "rssdescription"))
        (set 'SETUP:rss-link (CGI:get "rsslink"))
        (set 'SETUP:rss-copyright (CGI:get "rsscopyright"))
        (save "pages/setup.lsp" 'SETUP)
        (println {<html><body><h2>Setup saved</h2> <a href="index.cgi">continue</a></body></html>})
        (exit)))


;; request to display a page
(if (CGI:get "page")
    (begin
        (display-page (CGI:get "page"))
        (exit)))

;; request to show a list of pages sorted by last change date
(if (CGI:get "changes")
    (begin
        (set 'cnt 0)
        (dolist (fle (get-pages))
            (inc cnt)
            (set 'modified (nth 6 (file-info (append "pages/" fle))))
            (get-flags fle)
            (push (list modified fle L-flag P-flag M-flag T-flag S-flag O-flag C-flag) file-list))

        (set 'today (date (apply date-value (now))))
        (replace "  " today " ")
        (set 'today (slice today 0 10))
        (set 'files '())
        (set 'files '({</table>}))
        (dolist (rec (sort file-list))
            (set 'datstr (date (first rec)))
            (set 'fle (nth 1 rec))
            (set 'dfle (replace "_" (nth 1 rec) " "))
            (replace "  " datstr " ")
            (if (starts-with datstr today)
                (set 'mark {&nbsp;</td><td style="color: red;">*})
                (set 'mark {&nbsp;</td><td>}))
            (set 'datelst (parse datstr " "))
            (push (string 
                  {<tr><td>}
                  {&nbsp;<a href="index.cgi?page=} 
                  fle {">}  dfle {</a>}
                  {&nbsp;</td><td>&nbsp;} (nth 0 datelst)
                  {&nbsp;</td><td>&nbsp;} (nth 1 datelst)
                  {&nbsp;</td><td>&nbsp;} (nth 2 datelst)
                  {&nbsp;</td><td>&nbsp;} (nth 3 datelst)
                  {&nbsp;</td><td>&nbsp;} (nth 4 datelst)
                  mark
                  {</td>}
                  {<td><a href="index.cgi?edit=} 
                  fle {">&nbsp;edit&nbsp;</a></td>}
                  {<td style="color:#cc0; font-weight:bold">} (nth 2 rec) {</td>}
                  {<td style="color:#c00; font-weight:bold">} (nth 3 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 4 rec) {</td>}  
                  {<td style="color:#080; font-weight:bold">} (nth 5 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 6 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 7 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 8 rec) {</td>}
                  {</tr>} ) files))

        (push {<table style="border-width: 0px;">} files)
        (set 'body (join files "\r\n")) 
        (set 'body (append body (get-legend)
        {<br><small><font style="color:#888;">found } 
        (string cnt) { files</font></small>}))
        (set 'page-name "Last_Changes")
        (CGI:put-page SETUP:template)
        (exit)))

;; request to show a list with reference counts for each page
(if (CGI:get "refcount")
    (begin
        (dolist (fle (get-pages))
            (get-flags fle)
            (push (list (refcount fle) fle L-flag P-flag M-flag T-flag S-flag O-flag C-flag) file-list))

        (set 'cnt (length file-list))

        (set 'files '({</table>}))
        (dolist (rec (sort file-list))
            (set 'rcnt (first rec))
            (set 'fle (nth 1 rec))
            (set 'dfle (replace "_" (nth 1 rec) " "))
            (push (append 
                  {<tr><td align="right">&nbsp;} (string rcnt) {&nbsp;</td>}  
                  {<td>&nbsp;<a href="index.cgi?page=} fle {">} dfle {</a></td>}
                  {<td>&nbsp;<a href="index.cgi?edit=} fle {">edit</a></td>}
                  {<td style="color:#cc0; font-weight:bold">} (nth 2 rec) {</td>}
                  {<td style="color:#c00; font-weight:bold">} (nth 3 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 4 rec) {</td>}  
                  {<td style="color:#080; font-weight:bold">} (nth 5 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 6 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 7 rec) {</td>}
                  {<td style="color:#080; font-weight:bold">} (nth 8 rec) {</td>}
                  {</tr>}) files))

        (push {<table style="border-width: 0px;">} files)
        (set 'body (join files "\r\n"))            
        (set 'body (append body (get-legend)
        {<br><small><font style="color:#888;">found } 
        (string cnt) { files</font></small>}))
        (set 'page-name "Page_References")
        (CGI:put-page SETUP:template)
        (exit)))


;; request to show a list of all files with edit links
(if (CGI:get "index")
    (begin
        (set 'cnt 0)
        (set 'files '({</table>}))
        (dolist (fle (sort (get-pages) '>))
            (inc cnt)
            (get-flags fle)
            (set 'dfle fle) (replace "_" dfle " ")
            (push (string {<tr>}
                  {<td>&nbsp;<a href="index.cgi?page=} 
                  fle {">} dfle {</a>&nbsp;</td>}
                  {<td>&nbsp;<a href="index.cgi?edit=} fle {">edit</a></td>}
                  {<td style="color:#cc0; font-weight:bold">} L-flag {</td>}
                  {<td style="color:#c00; font-weight:bold">} P-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} M-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} T-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} S-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} O-flag {</td>}
                  {<td style="color:#080; font-weight:bold">} C-flag {</td>}
                  {</tr>}) files))

        (push {<table style="border-width: 0px;">} files)
        (set 'body (join files "\r\n"))            
        (set 'body (append body (get-legend) 
        {<br><small><font style="color:#888;">found } 
        (string cnt) { files</font></small>}))
        (set 'page-name "Alphabetical_Index")
        (CGI:put-page SETUP:template)
        (exit)))

;; request to show list of files with edit/backup/restore/delete links
(if (CGI:get "files")
    (files-table))


;; request to show a list of all files with serach pattern in title
(if (CGI:get "title")
    (begin
        (set 'body (search-title-table (CGI:get "title")))
        (set 'page-name (CGI:get "pagename")) 
        (if (not page-name) (set 'page-name "Title_Search"))
        (CGI:put-page SETUP:template)
        (exit)))


;; request to show list of file with pattern in content
;; serves search dialog
(if (CGI:get "search")
    (begin
        (set 'files (get-pages))
        (set 'no-files (length files))
        (set 'pattern (CGI:get "search"))

        (if (= pattern "") 
            (begin
                (display-page "Home")
                (exit)))

        (set 'file-list '())
        (dolist (fle files)
            (set 'content (get-content fle))

            ;; don't show the password option
            (replace {\[protected:.*\]} content "" 512)
            ;; don't show the post option
            (replace {\[post:.*\]} content "" 512)

            ;; don't search in locked pages
            (if (find {\[locked:.*\]} content 512) 
                (set 'pos nil)
                (set 'pos (find pattern content 0)))
            (if pos 
                (begin
                    (set 'str (slice content pos 64))
                    (push (list (append "..." str "...") fle) file-list))))

        (set 'files '({</table>}))
        (dolist (fle (sort file-list))
            (set 'cnt (first fle))
            (push (string 
                  {<tr><td>}
                  {&nbsp;<a href="index.cgi?page=} 
                  (last fle) {">} (replace "_" (last fle) " ") {</a>&nbsp;</td>}
                  {<td>&nbsp;} (first fle)   
                  {&nbsp;</td></tr>}) files))

        (if (empty? file-list) 
            (set 'result-text (string "Not found in " no-files " files.<br>"))
            (set 'result-text (append
            {<br><small><font style="color:#888;">found } 
            (string (length file-list)) { files</font></small>})))

        (if (not (empty? file-list))
            (push {<tr><td>&nbsp;Page</th><td>&nbsp;Position</td></tr>} files))

        (push {<table style="border-width: 0px;">} files)
        (set 'body (append (search-dialog pattern) (join files "\r\n")
              result-text))
        (set 'page-name "Search_Results")
        (CGI:put-page SETUP:template)
        (exit)))


;; request to show the first page with pattern in content
;; mainly used to find posts/comments by author post-time pattern
(if (CGI:get "find")
    (begin
    (set 'pattern (CGI:get "find"))
    (if (= pattern "")
        (begin
            (display-page "Home")
            (exit)))
    (set 'file-list (get-pages))
    (dolist (fle file-list)
        (set 'content (get-content fle))
        (if (find pattern content 0)
            (begin
                (display-page fle)
                (exit))))
    (display-page "Home")
    (exit)))


;; request for tag search
(if (CGI:get "tagsearch")
    (begin
      (set 'tag (CGI:get "tagsearch"))
      (if (null? tag)
    	  (begin (display-page "Home") (exit))
          (begin
            (setf match-tag? (fn (pg)
                               (let ((file (open (string "pages/" pg) "r")) (tagstr ""))
                                 (when file
                                       (and (search file {\[tags:(.*)\]} true 512) (setq tagstr $1))
                                       (close file))
                                 (find tag (map trim (parse tagstr ","))))))
            (set 'file-list (filter match-tag? (get-pages)))
            (set 'no-files (length file-list))
            (set 'files '("</table>"))
            (dolist (fle (sort file-list >))
                    (push (string
                           "<tr><td> &nbsp\;<a href=\"index.cgi?page="
                           fle "\">" (replace "_" fle " ") "</a>&nbsp\;</td></tr>")
                          files))
                         
            (if (empty? file-list)
                (set 'result-text "Not found.<br>")
                (set 'result-text (append
                                   "<br><small><font style=\"color:\#888\;\">found "
                                   (string no-files) { files</font></small>})))
            (if (not (empty? file-list))
                (push "<tr><th>&nbsp\;Page</th></tr>" files))

            (push {<table style="border-width: 0px;">} files)
            (set 'body (append (join files "\r\n") result-text))
            (set 'page-name (string "Pages tagged with \"" tag "\""))
            (CGI:put-page SETUP:template)
            (exit)))))
    

;; request to rename a page
(if (CGI:get "rename")
  (if (CGI:get "continue")
    (letn ( page-old           (CGI:get "rename")
            page-old-filename  (replace " " (dup page-old 1) "_")
            page-new           (CGI:get "new")
            page-new-filename  (replace " " (dup page-new 1) "_") )
      (if (= (length (trim page-new)) 0)
        (let ( body  (append {<p>You haven't entered any new name for the page } page-old {.</p>}
                             {<p>Try again on <a href="index.cgi?rename=} page-old
                             {">rename page</a>, go to }
                             {<a href="index.cgi?page=} page-old-filename {">} page-old {</a>, or }
                             {get back to <a href="index.cgi">Home</a>.</p>} ) )
          (set 'body body)
          (set 'page-name "Rename")
          (CGI:put-page SETUP:template)
          (exit))
        (begin
          (change-refs page-old page-new)
          (rename-file (append "pages/" page-old-filename) (append "pages/" page-new-filename))
          (display-page "Home")
          (exit))))
    (letn ( page-name  (CGI:get "rename")
            bdy  (append {<p>You can rename the page } page-name {.</p>}
                         {<form action="index.cgi" method="post">}
                         {<input type="hidden" name="rename" value="} page-name {" />}
                         {<input type="hidden" name="continue" value="" />}
                         {<p>New page name: <input type="text" name="new" /></p>}
                         {<p>Do you really want to rename the page ?&nbsp;}
                         {<input type="submit" name="submit" class="button" value="Yes" /></p>}
                         {</form>}
                         {<p>You can cancel this action and get back to <a href="index.cgi">Home</a>.</p>}) )
      (set 'body bdy)
      (set 'page-name "Rename")
      (CGI:put-page SETUP:template)
      (exit))))


;; take parameter as page name and display it
(set 'link (env "QUERY_STRING"))
(if (or (= link "") (not link))
    (display-page "Home")
    (display-page link))

(exit)


;; eof
